<form action="[element:search_url]" method="get" class="lineform">
  <div>
    <label for="query">Search</label>
    <input id="query" type="text" name="query">
  </div>
  <div>
    <a class="button" href="[element:search_url]">Clear</a>
    <input id="query" type="submit" value="Filter">
  </div>
</form>